import { CapitalizeSnakeCase, PickFirstTupleItemThatExtends } from './type-helpers';
export declare type GenericSchema = Record<string, any> & {
    introspection: Introspection;
    types: Record<string, any>;
};
export declare type OperationTypes = 'Query' | 'Mutation' | 'Subscription';
declare type RootOperationName<Schema extends GenericSchema | undefined, OperationType extends OperationTypes, capitalize extends boolean = true, IntrospectionSchema = Schema extends GenericSchema ? Schema['introspection']['__schema'] : never, IntrospectionProperty = `${Uncapitalize<OperationType>}Type`> = Schema extends GenericSchema ? IntrospectionProperty extends keyof IntrospectionSchema ? IntrospectionSchema[IntrospectionProperty] extends {
    name: string;
} ? capitalize extends true ? CapitalizeSnakeCase<IntrospectionSchema[IntrospectionProperty]['name']> : IntrospectionSchema[IntrospectionProperty]['name'] : never : never : never;
export declare type RootOperation<Schema extends GenericSchema | undefined, OperationType extends OperationTypes, OperationName = RootOperationName<Schema, OperationType>> = Schema extends GenericSchema ? OperationName extends string ? Schema['types'][OperationName] extends object ? Schema['types'][OperationName] : never : never : never;
export declare type OperationsOf<Schema extends GenericSchema | undefined, OperationType extends OperationTypes> = Omit<RootOperation<Schema | undefined, OperationType>, '__typename' | number | symbol>;
export declare type FieldArgs<Schema extends GenericSchema, OperationType extends OperationTypes, FieldName extends string, Suffix extends string = `_${CapitalizeSnakeCase<FieldName>}Args`> = Schema['types'][`${RootOperationName<Schema, OperationType>}${Suffix}`];
export declare type OperationRootTypeOf<Schema extends GenericSchema, OperationType extends OperationTypes> = SelectSingleType<Schema, {
    kind: 'OBJECT';
    name: RootOperationName<Schema, OperationType, false>;
}>;
export declare type SelectSingleType<S extends GenericSchema, Type extends TypeRef> = PickFirstTupleItemThatExtends<S['introspection']['__schema']['types'], Type extends {
    name: string;
} ? {
    name: Type['name'];
    kind: Type['kind'];
} : {
    kind: Type['kind'];
}>;
declare type MAX_RECURSION = 10;
declare type Enumerate<N extends number, Acc extends number[] = []> = Acc['length'] extends N ? Acc : Enumerate<N, [...Acc, Acc['length']]>;
declare type Pred = [never, ...Enumerate<MAX_RECURSION>];
export declare type ConcreteTypeOf<S extends GenericSchema, T extends TypeRef, D extends number = MAX_RECURSION> = [D] extends [0] ? any : T extends TypeRef ? T extends NonNullType ? ConcreteTypeOf<S, T['ofType'], Pred[D]> : T extends ListType ? ConcreteTypeOf<S, T['ofType'], Pred[D]> : T extends ObjectType | InterfaceType | UnionType ? SelectSingleType<S, T> : T : never;
export declare type Introspection = {
    __schema: {
        queryType: {
            name: string;
        } | null;
        mutationType: {
            name: string;
        } | null;
        subscriptionType: {
            name: string;
        } | null;
        types: Readonly<Array<TypeRef>>;
        directives: Readonly<[]>;
    };
};
export declare type InputObjectType = {
    kind: 'INPUT_OBJECT';
    name: string;
    ofType?: null;
    readonly inputFields?: Readonly<Array<FieldDefinition>>;
};
export declare type ListType = {
    kind: 'LIST';
    ofType: TypeRef;
};
export declare type NonNullType = {
    kind: 'NON_NULL';
    ofType: TypeRef;
};
export declare type ObjectType<Name extends string = string> = {
    kind: 'OBJECT';
    name: Name;
    interfaces?: Readonly<Array<unknown>>;
    ofType?: null;
    fields?: Readonly<Array<FieldDefinition>>;
};
export declare type ScalarType = {
    kind: 'SCALAR';
    name: string;
};
export declare type UnionType = {
    kind: 'UNION';
    name: string;
    ofType?: null;
    possibleTypes?: Readonly<Array<ObjectType | UnionType | InterfaceType>>;
};
export declare type EnumType = {
    kind: 'ENUM';
    name: string;
    ofType?: null;
    enumValues?: Readonly<Array<{
        name: string;
    }>>;
};
export declare type InterfaceType = {
    kind: 'INTERFACE';
    name: string;
    ofType?: null;
    fields?: Readonly<Array<FieldDefinition>>;
    possibleTypes?: Readonly<Array<ObjectType | UnionType | InterfaceType>>;
};
export declare type TypeRef = InputObjectType | ListType | NonNullType | ObjectType | ScalarType | UnionType | EnumType | InterfaceType;
export declare type FieldDefinition = {
    readonly name: string;
    readonly args?: ReadonlyArray<{
        name: string;
        type: TypeRef;
    }>;
    readonly type: TypeRef;
};
export declare const getIntrospectionType: <S extends GenericSchema, Type extends Partial<TypeRef>>(schema: S, type: Type) => TypeRef | undefined;
export declare const getRootOperationNames: <S extends GenericSchema | undefined, OperationType extends OperationTypes, Operations = OperationsOf<S, OperationType>>(schema: S, operationType: OperationType) => (keyof Operations)[];
export declare const getTypeFromRef: (schema: GenericSchema, typeRef?: TypeRef) => TypeRef;
export declare const getArgumentType: (name: String, definition: FieldDefinition) => TypeRef;
export declare const getFieldType: (schema: GenericSchema, fieldName: String, definition: TypeRef | FieldDefinition) => FieldDefinition | null;
export declare const getRootOperationNode: (schema: GenericSchema, opType: OperationTypes, rootOperation: string) => FieldDefinition;
export declare const getConcreteType: (schema: GenericSchema, definition: TypeRef | FieldDefinition) => TypeRef;
export declare const getGraphQLType: (schema: GenericSchema, type: TypeRef) => string;
export {};
//# sourceMappingURL=schema.d.ts.map